@@ -1,27 +1,37 @@
-/* $Id: SUPDrvTracer.cpp 143192 2021-03-09 16:53:01Z klaus $ */
+/* $Id: SUPDrvTracer.cpp 153224 2022-08-22 17:43:14Z klaus $ */
 /** @file
  * VBoxDrv - The VirtualBox Support Driver - Tracer Interface.
  */
 
 /*
- * Copyright (C) 2012-2020 Oracle Corporation
+ * Copyright (C) 2012-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -1097,6 +1107,7 @@ SUPR0DECL(int) SUPR0TracerRegisterDrv(PSUPDRVSESSION pSession, PVTGOBJHDR pVtgHd
 
     return rc;
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerRegisterDrv);
 
 
 /**
@@ -1129,6 +1140,7 @@ SUPR0DECL(void) SUPR0TracerDeregisterDrv(PSUPDRVSESSION pSession)
      */
     supdrvTracerProcessZombies(pDevExt);
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerDeregisterDrv);
 
 
 /**
@@ -1175,6 +1187,7 @@ SUPR0DECL(int) SUPR0TracerRegisterModule(void *hMod, PVTGOBJHDR pVtgHdr)
 
     return rc;
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerRegisterModule);
 
 
 /**
@@ -1274,6 +1287,7 @@ SUPR0DECL(int) SUPR0TracerRegisterImpl(void *hMod, PSUPDRVSESSION pSession, PCSU
     return rc;
 
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerRegisterImpl);
 
 
 /**
@@ -1459,6 +1473,7 @@ SUPR0DECL(int) SUPR0TracerDeregisterImpl(void *hMod, PSUPDRVSESSION pSession)
 
     return rc;
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerDeregisterImpl);
 
 
 /*
@@ -1505,6 +1520,7 @@ supdrvTracerProbeFireStub:                                              \n\
  )
 # endif
 #endif
+SUPR0_EXPORT_SYMBOL(SUPR0TracerFireProbe);
 
 
 /**
@@ -2229,6 +2245,7 @@ SUPR0DECL(void) SUPR0TracerUmodProbeFire(PSUPDRVSESSION pSession, PSUPDRVTRACERU
 
     supdrvTracerUmodProbeFire(pSession->pDevExt, pSession, pCtx);
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerUmodProbeFire);
 
 
 void  VBOXCALL  supdrvIOCtl_TracerUmodProbeFire(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, PSUPDRVTRACERUSRCTX pCtx)
@@ -2468,7 +2485,9 @@ void VBOXCALL supdrvTracerTerm(PSUPDRVDEVEXT pDevExt)
     LOG_TRACER(("supdrvTracerTerm\n"));
 
     supdrvTracerRemoveAllProviders(pDevExt);
-
+#ifdef VBOX_WITH_NATIVE_DTRACE
+    supdrvDTraceFini();
+#endif
     RTSemFastMutexDestroy(pDevExt->mtxTracer);
     pDevExt->mtxTracer = NIL_RTSEMFASTMUTEX;
     LOG_TRACER(("supdrvTracerTerm: Done\n"));
