@@ -1,27 +1,37 @@
-/* $Id: SUPLibAll.cpp 135976 2020-02-04 10:35:17Z bird $ */
+/* $Id: SUPLibAll.cpp 153224 2022-08-22 17:43:14Z klaus $ */
 /** @file
  * VirtualBox Support Library - All Contexts Code.
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -40,10 +50,13 @@
 # include <iprt/asm-amd64-x86.h>
 #endif
 #include <iprt/errcore.h>
+#if defined(IN_RING0) && defined(RT_OS_LINUX)
+# include "SUPDrvInternal.h"
+#endif
+
 
 
 #if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
-
 /**
  * The slow case for SUPReadTsc where we need to apply deltas.
  *
@@ -217,6 +230,10 @@ SUPDECL(uint64_t) SUPReadTscWithDelta(PSUPGLOBALINFOPAGE pGip)
     AssertMsgFailed(("iGipCpu=%d (%#x) cCpus=%d fGetGipCpu=%#x\n", iGipCpu, iGipCpu, pGip->cCpus, pGip->fGetGipCpu));
     return uTsc;
 }
+# ifdef SUPR0_EXPORT_SYMBOL
+SUPR0_EXPORT_SYMBOL(SUPReadTscWithDelta);
+# endif
+#endif /* RT_ARCH_AMD64 || RT_ARCH_X86 */
 
 
 /**
@@ -229,6 +246,7 @@ DECLINLINE(uint16_t) supGetGipCpuIndex(PSUPGLOBALINFOPAGE pGip)
 {
     uint16_t iGipCpu;
 #ifdef IN_RING3
+# if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
     if (pGip->fGetGipCpu & SUPGIPGETCPU_IDTR_LIMIT_MASK_MAX_SET_CPUS)
     {
         /* Storing the IDTR is normally very fast. */
@@ -263,6 +281,15 @@ DECLINLINE(uint16_t) supGetGipCpuIndex(PSUPGLOBALINFOPAGE pGip)
         uint8_t idApic = ASMGetApicId();
         iGipCpu = pGip->aiCpuFromApicId[idApic];
     }
+
+# else
+    int iCpuSet = RTMpCpuIdToSetIndex(RTMpCpuId());
+    if (RT_LIKELY((unsigned)iCpuSet < RT_ELEMENTS(pGip->aiCpuFromCpuSetIdx)))
+        iGipCpu = pGip->aiCpuFromCpuSetIdx[iCpuSet];
+    else
+        iGipCpu = UINT16_MAX;
+# endif
+
 #elif defined(IN_RING0)
     /* Ring-0: Use use RTMpCpuId() (disables cli to avoid host OS assertions about unsafe CPU number usage). */
     RTCCUINTREG uFlags  = ASMIntDisableFlags();
@@ -280,6 +307,7 @@ DECLINLINE(uint16_t) supGetGipCpuIndex(PSUPGLOBALINFOPAGE pGip)
         iGipCpu = pGip->aiCpuFromCpuSetIdx[iCpuSet];
     else
         iGipCpu = UINT16_MAX;
+
 #else
 # error "IN_RING3, IN_RC or IN_RING0 must be defined!"
 #endif
@@ -294,7 +322,7 @@ DECLINLINE(uint16_t) supGetGipCpuIndex(PSUPGLOBALINFOPAGE pGip)
  * @param   pGip        The GIP.
  * @internal
  */
-SUPDECL(uint64_t) SUPGetTscDeltaSlow(PSUPGLOBALINFOPAGE pGip)
+SUPDECL(int64_t) SUPGetTscDeltaSlow(PSUPGLOBALINFOPAGE pGip)
 {
     uint16_t iGipCpu = supGetGipCpuIndex(pGip);
     if (RT_LIKELY(iGipCpu < pGip->cCpus))
@@ -308,6 +336,22 @@ SUPDECL(uint64_t) SUPGetTscDeltaSlow(PSUPGLOBALINFOPAGE pGip)
 }
 
 
+/**
+ * SLow path in SUPGetGipCpuPtr, don't call directly.
+ *
+ * @returns Pointer to the CPU entry for the caller, NULL on failure.
+ * @param   pGip        The GIP.
+ */
+SUPDECL(PSUPGIPCPU) SUPGetGipCpuPtrForAsyncMode(PSUPGLOBALINFOPAGE pGip)
+{
+    uint16_t iGipCpu = supGetGipCpuIndex(pGip);
+    if (RT_LIKELY(iGipCpu < pGip->cCpus))
+        return &pGip->aCPUs[iGipCpu];
+    AssertFailed();
+    return NULL;
+}
+
+
 /**
  * Slow path in SUPGetCpuHzFromGip, don't call directly.
  *
@@ -325,6 +369,7 @@ SUPDECL(uint64_t) SUPGetCpuHzFromGipForAsyncMode(PSUPGLOBALINFOPAGE pGip)
 }
 
 
+
 /**
  * Worker for SUPIsTscFreqCompatible().
  *
@@ -382,5 +427,3 @@ SUPDECL(bool) SUPIsTscFreqCompatible(uint64_t uCpuHz, uint64_t *puGipCpuHz, bool
     return fCompat;
 }
 
-#endif /* RT_ARCH_AMD64 || RT_ARCH_X86 */
-
