@@ -1,27 +1,37 @@
-/* $Id: logrel.cpp 135976 2020-02-04 10:35:17Z bird $ */
+/* $Id: logrel.cpp 153224 2022-08-22 17:43:14Z klaus $ */
 /** @file
  * Runtime VBox - Release Logger.
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -31,95 +41,6 @@
 #include <iprt/log.h>
 #include "internal/iprt.h"
 
-#ifndef IN_RC
-# include <iprt/alloc.h>
-# include <iprt/process.h>
-# include <iprt/semaphore.h>
-# include <iprt/thread.h>
-# include <iprt/mp.h>
-#endif
-#ifdef IN_RING3
-# include <iprt/file.h>
-# include <iprt/path.h>
-#endif
-#include <iprt/time.h>
-#include <iprt/asm.h>
-#include <iprt/assert.h>
-#include <iprt/errcore.h>
-#include <iprt/param.h>
-
-#include <iprt/stdarg.h>
-#include <iprt/string.h>
-#include <iprt/ctype.h>
-#ifdef IN_RING3
-# include <iprt/alloca.h>
-# include <stdio.h>
-#endif
-
-
-/*********************************************************************************************************************************
-*   Global Variables                                                                                                             *
-*********************************************************************************************************************************/
-#ifdef IN_RC
-/** Default release logger instance. */
-extern "C" DECLIMPORT(RTLOGGERRC)   g_RelLogger;
-#else /* !IN_RC */
-/** Default release logger instance. */
-static PRTLOGGER                    g_pRelLogger;
-#endif /* !IN_RC */
-
-
-RTDECL(PRTLOGGER)   RTLogRelGetDefaultInstance(void)
-{
-#ifdef IN_RC
-    return &g_RelLogger;
-#else /* !IN_RC */
-    return g_pRelLogger;
-#endif /* !IN_RC */
-}
-RT_EXPORT_SYMBOL(RTLogRelGetDefaultInstance);
-
-
-RTDECL(PRTLOGGER)   RTLogRelGetDefaultInstanceEx(uint32_t fFlagsAndGroup)
-{
-#ifdef IN_RC
-    PRTLOGGER pLogger = &g_RelLogger;
-#else /* !IN_RC */
-    PRTLOGGER pLogger = g_pRelLogger;
-#endif /* !IN_RC */
-    if (pLogger)
-    {
-        if (pLogger->fFlags & RTLOGFLAGS_DISABLED)
-            pLogger = NULL;
-        else
-        {
-            uint16_t const fFlags = RT_LO_U16(fFlagsAndGroup);
-            uint16_t const iGroup = RT_HI_U16(fFlagsAndGroup);
-            if (   iGroup != UINT16_MAX
-                 && (   (pLogger->afGroups[iGroup < pLogger->cGroups ? iGroup : 0] & (fFlags | (uint32_t)RTLOGGRPFLAGS_ENABLED))
-                     != (fFlags | (uint32_t)RTLOGGRPFLAGS_ENABLED)))
-            pLogger = NULL;
-        }
-    }
-    return pLogger;
-}
-RT_EXPORT_SYMBOL(RTLogRelGetDefaultInstanceEx);
-
-
-#ifndef IN_RC
-/**
- * Sets the default logger instance.
- *
- * @returns iprt status code.
- * @param   pLogger     The new default release logger instance.
- */
-RTDECL(PRTLOGGER) RTLogRelSetDefaultInstance(PRTLOGGER pLogger)
-{
-    return ASMAtomicXchgPtrT(&g_pRelLogger, pLogger, PRTLOGGER);
-}
-RT_EXPORT_SYMBOL(RTLogRelSetDefaultInstance);
-#endif /* !IN_RC */
-
 
 /**
  * Write to a logger instance, defaulting to the release one.
