@@ -1,27 +1,37 @@
-/* $Id: SUPDrv-linux.c 151848 2022-06-13 20:36:29Z vgalitsy $ */
+/* $Id: SUPDrv-linux.c 153224 2022-08-22 17:43:14Z klaus $ */
 /** @file
  * VBoxDrv - The VirtualBox Support Driver - Linux specifics.
  */
 
 /*
- * Copyright (C) 2006-2020 Oracle Corporation
+ * Copyright (C) 2006-2022 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -100,6 +110,21 @@
 #endif
 
 
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
+#if RTLNX_VER_MIN(5,0,0)
+/** Wrapper module list entry. */
+typedef struct SUPDRVLNXMODULE
+{
+    RTLISTNODE      ListEntry;
+    struct module  *pModule;
+} SUPDRVLNXMODULE;
+/** Pointer to a wrapper module list entry. */
+typedef SUPDRVLNXMODULE *PSUPDRVLNXMODULE;
+#endif
+
+
 /*********************************************************************************************************************************
 *   Internal Functions                                                                                                           *
 *********************************************************************************************************************************/
@@ -126,6 +151,10 @@ static int  VBoxDrvResume(struct platform_device *pDev);
 # endif
 static void VBoxDevRelease(struct device *pDev);
 #endif
+#if RTLNX_VER_MIN(5,0,0)
+static int  supdrvLinuxLdrModuleNotifyCallback(struct notifier_block *pBlock,
+                                               unsigned long uModuleState, void *pvModule);
+#endif
 
 
 /*********************************************************************************************************************************
@@ -225,6 +254,7 @@ static struct miscdevice gMiscDeviceUsr =
 
 
 #ifdef VBOX_WITH_SUSPEND_NOTIFICATION
+
 # if RTLNX_VER_MIN(2,6,30)
 static struct dev_pm_ops gPlatformPMOps =
 {
@@ -260,14 +290,29 @@ static struct platform_device gPlatformDevice =
         .release = VBoxDevRelease
     }
 };
+
 #endif /* VBOX_WITH_SUSPEND_NOTIFICATION */
 
+#if RTLNX_VER_MIN(5,0,0)
+/** Module load/unload notification registration record. */
+static struct notifier_block    g_supdrvLinuxModuleNotifierBlock =
+{
+    .notifier_call = supdrvLinuxLdrModuleNotifyCallback,
+    .priority      = 0
+};
+/** Spinlock protecting g_supdrvLinuxWrapperModuleList. */
+static spinlock_t               g_supdrvLinuxWrapperModuleSpinlock;
+/** List of potential wrapper modules (PSUPDRVLNXMODULE). */
+static RTLISTANCHOR             g_supdrvLinuxWrapperModuleList;
+#endif
 
-DECLINLINE(RTUID) vboxdrvLinuxUid(void)
+
+/** Get the kernel UID for the current process. */
+DECLINLINE(RTUID) vboxdrvLinuxKernUid(void)
 {
 #if RTLNX_VER_MIN(2,6,29)
 # if RTLNX_VER_MIN(3,5,0)
-    return from_kuid(current_user_ns(), current->cred->uid);
+    return __kuid_val(current->cred->uid);
 # else
     return current->cred->uid;
 # endif
@@ -276,11 +321,13 @@ DECLINLINE(RTUID) vboxdrvLinuxUid(void)
 #endif
 }
 
-DECLINLINE(RTGID) vboxdrvLinuxGid(void)
+
+/** Get the kernel GID for the current process. */
+DECLINLINE(RTGID) vboxdrvLinuxKernGid(void)
 {
 #if RTLNX_VER_MIN(2,6,29)
 # if RTLNX_VER_MIN(3,5,0)
-    return from_kgid(current_user_ns(), current->cred->gid);
+    return __kgid_val(current->cred->gid);
 # else
     return current->cred->gid;
 # endif
@@ -289,18 +336,23 @@ DECLINLINE(RTGID) vboxdrvLinuxGid(void)
 #endif
 }
 
-DECLINLINE(RTUID) vboxdrvLinuxEuid(void)
+
+#ifdef VBOX_WITH_HARDENING
+/** Get the effective UID within the current user namespace. */
+DECLINLINE(RTUID) vboxdrvLinuxEuidInNs(void)
 {
-#if RTLNX_VER_MIN(2,6,29)
-# if RTLNX_VER_MIN(3,5,0)
+# if RTLNX_VER_MIN(2,6,29)
+#  if RTLNX_VER_MIN(3,5,0)
     return from_kuid(current_user_ns(), current->cred->euid);
-# else
+#  else
     return current->cred->euid;
-# endif
-#else
+#  endif
+# else
     return current->euid;
-#endif
+# endif
 }
+#endif
+
 
 /**
  * Initialize module.
@@ -311,10 +363,15 @@ static int __init VBoxDrvLinuxInit(void)
 {
     int       rc;
 
+#if RTLNX_VER_MIN(5,0,0)
+    spin_lock_init(&g_supdrvLinuxWrapperModuleSpinlock);
+    RTListInit(&g_supdrvLinuxWrapperModuleList);
+#endif
+
     /*
      * Check for synchronous/asynchronous TSC mode.
      */
-    printk(KERN_DEBUG "vboxdrv: Found %u processor cores\n", (unsigned)RTMpGetOnlineCount());
+    printk(KERN_DEBUG "vboxdrv: Found %u processor cores/threads\n", (unsigned)RTMpGetOnlineCount());
     rc = misc_register(&gMiscDeviceSys);
     if (rc)
     {
@@ -362,6 +419,16 @@ static int __init VBoxDrvLinuxInit(void)
                     if (rc == 0)
 #endif
                     {
+#if RTLNX_VER_MIN(5,0,0)
+                        /*
+                         * Register the module notifier.
+                         */
+                        int rc2 = register_module_notifier(&g_supdrvLinuxModuleNotifierBlock);
+                        if (rc2)
+                            printk(KERN_WARNING "vboxdrv: failed to register module notifier! rc2=%d\n", rc2);
+#endif
+
+
                         printk(KERN_INFO "vboxdrv: TSC mode is %s, tentative frequency %llu Hz\n",
                                SUPGetGIPModeName(g_DevExt.pGip), g_DevExt.pGip->u64CpuHz);
                         LogFlow(("VBoxDrv::ModuleInit returning %#x\n", rc));
@@ -407,6 +474,26 @@ static void __exit VBoxDrvLinuxUnload(void)
     platform_driver_unregister(&gPlatformDriver);
 #endif
 
+#if RTLNX_VER_MIN(5,0,0)
+    /*
+     * Kick the list of potential wrapper modules.
+     */
+    unregister_module_notifier(&g_supdrvLinuxModuleNotifierBlock);
+
+    spin_lock(&g_supdrvLinuxWrapperModuleSpinlock);
+    while (!RTListIsEmpty(&g_supdrvLinuxWrapperModuleList))
+    {
+        PSUPDRVLNXMODULE pCur = RTListRemoveFirst(&g_supdrvLinuxWrapperModuleList, SUPDRVLNXMODULE, ListEntry);
+        spin_unlock(&g_supdrvLinuxWrapperModuleSpinlock);
+
+        pCur->pModule = NULL;
+        RTMemFree(pCur);
+
+        spin_lock(&g_supdrvLinuxWrapperModuleSpinlock);
+    }
+    spin_unlock(&g_supdrvLinuxWrapperModuleSpinlock);
+#endif
+
     /*
      * I Don't think it's possible to unload a driver which processes have
      * opened, at least we'll blindly assume that here.
@@ -440,9 +527,9 @@ static int vboxdrvLinuxCreateCommon(struct inode *pInode, struct file *pFilp, bo
      * Only root is allowed to access the unrestricted device, enforce it!
      */
     if (   fUnrestricted
-        && vboxdrvLinuxEuid() != 0 /* root */ )
+        && vboxdrvLinuxEuidInNs() != 0 /* root */ )
     {
-        Log(("VBoxDrvLinuxCreate: euid=%d, expected 0 (root)\n", vboxdrvLinuxEuid()));
+        Log(("VBoxDrvLinuxCreate: euid=%d, expected 0 (root)\n", vboxdrvLinuxEuidInNs()));
         return -EPERM;
     }
 #endif /* VBOX_WITH_HARDENING */
@@ -453,8 +540,8 @@ static int vboxdrvLinuxCreateCommon(struct inode *pInode, struct file *pFilp, bo
     rc = supdrvCreateSession(&g_DevExt, true /* fUser */, fUnrestricted, &pSession);
     if (!rc)
     {
-        pSession->Uid = vboxdrvLinuxUid();
-        pSession->Gid = vboxdrvLinuxGid();
+        pSession->Uid = vboxdrvLinuxKernUid();
+        pSession->Gid = vboxdrvLinuxKernGid();
     }
 
     pFilp->private_data = pSession;
@@ -739,13 +826,13 @@ int VBOXCALL SUPDrvLinuxIDC(uint32_t uReq, PSUPDRVIDCREQHDR pReq)
     /*
      * Some quick validations.
      */
-    if (RT_UNLIKELY(!VALID_PTR(pReq)))
+    if (RT_UNLIKELY(!RT_VALID_PTR(pReq)))
         return VERR_INVALID_POINTER;
 
     pSession = pReq->pSession;
     if (pSession)
     {
-        if (RT_UNLIKELY(!VALID_PTR(pSession)))
+        if (RT_UNLIKELY(!RT_VALID_PTR(pSession)))
             return VERR_INVALID_PARAMETER;
         if (RT_UNLIKELY(pSession->pDevExt != &g_DevExt))
             return VERR_INVALID_PARAMETER;
@@ -758,10 +845,183 @@ int VBOXCALL SUPDrvLinuxIDC(uint32_t uReq, PSUPDRVIDCREQHDR pReq)
      */
     return supdrvIDC(uReq, &g_DevExt, pSession, pReq);
 }
-
 EXPORT_SYMBOL(SUPDrvLinuxIDC);
 
 
+#if RTLNX_VER_MIN(5,0,0)
+
+/**
+ * Checks if the given module is one of our potential wrapper modules or not.
+ */
+static bool supdrvLinuxLdrIsPotentialWrapperModule(struct module const *pModule)
+{
+    if (   pModule
+        && strncmp(pModule->name, RT_STR_TUPLE("vbox_")) == 0)
+        return true;
+    return false;
+}
+
+/**
+ * Called when a kernel module changes state.
+ *
+ * We use this to listen for wrapper modules being loaded, since some evil
+ * bugger removed the find_module() export in 5.13.
+ */
+static int supdrvLinuxLdrModuleNotifyCallback(struct notifier_block *pBlock, unsigned long uModuleState, void *pvModule)
+{
+    struct module *pModule = (struct module *)pvModule;
+    switch (uModuleState)
+    {
+        case MODULE_STATE_UNFORMED: /* Setting up the module... */
+            break;
+
+        /*
+         * The module is about to have its ctors & init functions called.
+         *
+         * Add anything that looks like a wrapper module to our tracker list.
+         */
+        case MODULE_STATE_COMING:
+            if (supdrvLinuxLdrIsPotentialWrapperModule(pModule))
+            {
+                PSUPDRVLNXMODULE pTracker = (PSUPDRVLNXMODULE)RTMemAlloc(sizeof(*pTracker));
+                if (pTracker)
+                {
+                    pTracker->pModule = pModule;
+                    spin_lock(&g_supdrvLinuxWrapperModuleSpinlock);
+                    RTListPrepend(&g_supdrvLinuxWrapperModuleList, &pTracker->ListEntry);
+                    spin_unlock(&g_supdrvLinuxWrapperModuleSpinlock);
+                }
+            }
+            break;
+
+        case MODULE_STATE_LIVE:
+            break;
+
+        /*
+         * The module has been uninited and is going away.
+         *
+         * Remove the tracker entry for the module, if we have one.
+         */
+        case MODULE_STATE_GOING:
+        {
+            PSUPDRVLNXMODULE pCur;
+            spin_lock(&g_supdrvLinuxWrapperModuleSpinlock);
+            RTListForEach(&g_supdrvLinuxWrapperModuleList, pCur, SUPDRVLNXMODULE, ListEntry)
+            {
+                if (pCur->pModule == pModule)
+                {
+                    RTListNodeRemove(&pCur->ListEntry);
+                    spin_unlock(&g_supdrvLinuxWrapperModuleSpinlock);
+
+                    pCur->pModule = NULL;
+                    RTMemFree(pCur);
+
+                    spin_lock(&g_supdrvLinuxWrapperModuleSpinlock); /* silly */
+                    break;
+                }
+            }
+            spin_unlock(&g_supdrvLinuxWrapperModuleSpinlock);
+            break;
+        }
+    }
+    RT_NOREF(pBlock);
+    return NOTIFY_OK;
+}
+
+/**
+ * Replacement for find_module() that's no longer exported with 5.13.
+ */
+static struct module *supdrvLinuxLdrFindModule(const char *pszLnxModName)
+{
+    PSUPDRVLNXMODULE pCur;
+
+    spin_lock(&g_supdrvLinuxWrapperModuleSpinlock);
+    RTListForEach(&g_supdrvLinuxWrapperModuleList, pCur, SUPDRVLNXMODULE, ListEntry)
+    {
+        struct module * const pModule = pCur->pModule;
+        if (   pModule
+            && strcmp(pszLnxModName, pModule->name) == 0)
+        {
+            spin_unlock(&g_supdrvLinuxWrapperModuleSpinlock);
+            return pModule;
+        }
+    }
+    spin_unlock(&g_supdrvLinuxWrapperModuleSpinlock);
+    return NULL;
+}
+
+#endif /* >= 5.0.0 */
+
+
+/**
+ * Used by native wrapper modules, forwarding to supdrvLdrRegisterWrappedModule
+ * with device extension prepended to the argument list.
+ */
+SUPR0DECL(int)  SUPDrvLinuxLdrRegisterWrappedModule(PCSUPLDRWRAPPEDMODULE pWrappedModInfo,
+                                                    const char *pszLnxModName, void **phMod)
+{
+    AssertPtrReturn(pszLnxModName, VERR_INVALID_POINTER);
+    AssertReturn(*pszLnxModName, VERR_INVALID_NAME);
+
+    /* Locate the module structure for the caller so can later reference
+       and dereference it to prevent unloading while it is being used.
+
+       Before Linux v5.9 this could be done by address (__module_address()
+       or __module_text_address()), but someone (guess who) apparently on
+       a mission to make life miserable for out-of-tree modules or something,
+       decided it was only used by build-in code and unexported both of them.
+
+       I could find no init callouts getting a struct module pointer either,
+       nor any module name hint anywhere I could see.  So, we're left with
+       hardcoding the module name via the compiler and pass it along to
+       SUPDrv so we can call find_module() here.
+
+       Sigh^2.
+
+       Update 5.13:
+       The find_module() and module_mutex symbols are no longer exported,
+       probably the doing of the same evil bugger mentioned above.  So, we now
+       register a module notification callback and track the modules we're
+       interested in that way. */
+
+#if RTLNX_VER_MIN(5,0,0)
+    struct module *pLnxModule = supdrvLinuxLdrFindModule(pszLnxModName);
+    if (pLnxModule)
+        return supdrvLdrRegisterWrappedModule(&g_DevExt, pWrappedModInfo, pLnxModule, phMod);
+    printk("vboxdrv: supdrvLinuxLdrFindModule(%s) failed in SUPDrvLinuxLdrRegisterWrappedModule!\n", pszLnxModName);
+    return VERR_MODULE_NOT_FOUND;
+
+#elif RTLNX_VER_MIN(2,6,30)
+    if (mutex_lock_interruptible(&module_mutex) == 0)
+    {
+        struct module *pLnxModule = find_module(pszLnxModName);
+        mutex_unlock(&module_mutex);
+        if (pLnxModule)
+            return supdrvLdrRegisterWrappedModule(&g_DevExt, pWrappedModInfo, pLnxModule, phMod);
+        printk("vboxdrv: find_module(%s) failed in SUPDrvLinuxLdrRegisterWrappedModule!\n", pszLnxModName);
+        return VERR_MODULE_NOT_FOUND;
+    }
+    return VERR_INTERRUPTED;
+
+#else
+    printk("vboxdrv: wrapper modules are not supported on 2.6.29 and earlier. sorry.\n");
+    return VERR_NOT_SUPPORTED;
+#endif
+}
+EXPORT_SYMBOL(SUPDrvLinuxLdrRegisterWrappedModule);
+
+
+/**
+ * Used by native wrapper modules, forwarding to supdrvLdrDeregisterWrappedModule
+ * with device extension prepended to the argument list.
+ */
+SUPR0DECL(int) SUPDrvLinuxLdrDeregisterWrappedModule(PCSUPLDRWRAPPEDMODULE pWrappedModInfo, void **phMod)
+{
+    return supdrvLdrDeregisterWrappedModule(&g_DevExt, pWrappedModInfo, phMod);
+}
+EXPORT_SYMBOL(SUPDrvLinuxLdrDeregisterWrappedModule);
+
+
 RTCCUINTREG VBOXCALL supdrvOSChangeCR4(RTCCUINTREG fOrMask, RTCCUINTREG fAndMask)
 {
 #if RTLNX_VER_MIN(5,8,0)
@@ -1251,6 +1511,28 @@ int  VBOXCALL   supdrvOSLdrQuerySymbol(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pI
 }
 
 
+void VBOXCALL   supdrvOSLdrRetainWrapperModule(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage)
+{
+    struct module *pLnxMod = (struct module *)pImage->pvWrappedNative;
+    Assert(!pImage->fLnxWrapperRef);
+    AssertReturnVoid(pLnxMod);
+    pImage->fLnxWrapperRef = try_module_get(pLnxMod);
+    RT_NOREF(pDevExt);
+}
+
+
+void VBOXCALL   supdrvOSLdrReleaseWrapperModule(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage)
+{
+    if (pImage->fLnxWrapperRef)
+    {
+        struct module *pLnxMod = (struct module *)pImage->pvWrappedNative;
+        pImage->fLnxWrapperRef = false;
+        module_put(pLnxMod);
+    }
+    RT_NOREF(pDevExt);
+}
+
+
 #ifdef SUPDRV_WITH_MSR_PROBER
 
 int VBOXCALL    supdrvOSMsrProberRead(uint32_t uMsr, RTCPUID idCpu, uint64_t *puValue)
@@ -1419,6 +1701,7 @@ SUPR0DECL(int) SUPR0HCPhysToVirt(RTHCPHYS HCPhys, void **ppv)
     *ppv = phys_to_virt(HCPhys);
     return VINF_SUCCESS;
 }
+SUPR0_EXPORT_SYMBOL(SUPR0HCPhysToVirt);
 
 
 RTDECL(int) SUPR0PrintfV(const char *pszFormat, va_list va)
@@ -1434,6 +1717,7 @@ RTDECL(int) SUPR0PrintfV(const char *pszFormat, va_list va)
     IPRT_LINUX_RESTORE_EFL_AC();
     return 0;
 }
+SUPR0_EXPORT_SYMBOL(SUPR0PrintfV);
 
 
 SUPR0DECL(uint32_t) SUPR0GetKernelFeatures(void)
@@ -1453,6 +1737,7 @@ SUPR0DECL(uint32_t) SUPR0GetKernelFeatures(void)
 #endif
     return fFlags;
 }
+SUPR0_EXPORT_SYMBOL(SUPR0GetKernelFeatures);
 
 
 SUPR0DECL(bool) SUPR0FpuBegin(bool fCtxHook)
@@ -1493,6 +1778,7 @@ SUPR0DECL(bool) SUPR0FpuBegin(bool fCtxHook)
     return false;
 #endif
 }
+SUPR0_EXPORT_SYMBOL(SUPR0FpuBegin);
 
 
 SUPR0DECL(void) SUPR0FpuEnd(bool fCtxHook)
@@ -1508,6 +1794,7 @@ SUPR0DECL(void) SUPR0FpuEnd(bool fCtxHook)
     kernel_fpu_end();
 #endif
 }
+SUPR0_EXPORT_SYMBOL(SUPR0FpuEnd);
 
 
 int VBOXCALL    supdrvOSGetCurrentGdtRw(RTHCUINTPTR *pGdtRw)
